@@ -11,15 +11,11 @@
  * @obj: a pointer to the #AccessibleApplication on which to operate.
  *
  * Increment the reference count for an #AccessibleApplication.
- *
- * Returns: (no return code implemented yet).
- *
  **/
-int
+void
 AccessibleApplication_ref (AccessibleApplication *obj)
 {
-  Accessibility_Application_ref (*obj, spi_ev ());
-  return 0;
+  cspi_object_ref (obj);
 }
 
 /**
@@ -27,15 +23,11 @@ AccessibleApplication_ref (AccessibleApplication *obj)
  * @obj: a pointer to the #AccessibleApplication object on which to operate.
  *
  * Decrement the reference count for an #AccessibleApplication.
- *
- * Returns: (no return code implemented yet).
- *
  **/
-int
+void
 AccessibleApplication_unref (AccessibleApplication *obj)
 {
-  Accessibility_Application_unref (*obj, spi_ev ());
-  return 0;
+  cspi_object_unref (obj);
 }
 
 /**
@@ -46,12 +38,21 @@ AccessibleApplication_unref (AccessibleApplication *obj)
  *
  * Returns: a UTF-8 string indicating which UI toolkit is
  *          used by an application.
- *
  **/
 char *
 AccessibleApplication_getToolkitName (AccessibleApplication *obj)
 {
-  return Accessibility_Application__get_toolkitName (*obj, spi_ev ());
+  char *retval;
+
+  cspi_return_val_if_fail (obj != NULL, NULL);
+
+  retval =
+    Accessibility_Application__get_toolkitName (CSPI_OBJREF (obj),
+						cspi_ev ());
+
+  cspi_return_val_if_ev ("toolkitName", NULL);
+
+  return retval;
 }
 
 /**
@@ -63,12 +64,21 @@ AccessibleApplication_getToolkitName (AccessibleApplication *obj)
  *
  * Returns: a UTF-8 string indicating the application's
  *          at-spi version.
- *
  **/
 char *
 AccessibleApplication_getVersion (AccessibleApplication *obj)
 {
-  return Accessibility_Application__get_version (*obj, spi_ev ());
+  char *retval;
+
+  cspi_return_val_if_fail (obj != NULL, NULL);
+
+  retval =
+    Accessibility_Application__get_version (CSPI_OBJREF (obj),
+					    cspi_ev ());
+
+  cspi_return_val_if_ev ("getVersion", NULL);
+
+  return retval;
 }
 
 /**
@@ -81,11 +91,20 @@ AccessibleApplication_getVersion (AccessibleApplication *obj)
  *
  * Returns: a unique #long integer associated with the application
  *          by the Registry, or 0 if the application is not registered.
-**/
+ **/
 long
 AccessibleApplication_getID (AccessibleApplication *obj)
 {
-  return Accessibility_Application__get_id (*obj, spi_ev ());
+  long retval;
+
+  cspi_return_val_if_fail (obj != NULL, 0);
+
+  retval = Accessibility_Application__get_id (CSPI_OBJREF (obj),
+					      cspi_ev ());
+
+  cspi_return_val_if_ev ("get_id", 0);
+
+  return retval;
 }
 
 /**
@@ -99,7 +118,7 @@ AccessibleApplication_getID (AccessibleApplication *obj)
  * Returns: #TRUE if the application was paused successfully, #FALSE otherwise.
  *
  **/
-boolean
+SPIBoolean
 AccessibleApplication_pause (AccessibleApplication *obj)
 {
   return FALSE;
@@ -115,10 +134,8 @@ AccessibleApplication_pause (AccessibleApplication *obj)
  * Returns: #TRUE if application processing resumed successfully, #FALSE otherwise.
  *
  **/
-boolean
+SPIBoolean
 AccessibleApplication_resume (AccessibleApplication *obj)
 {
   return FALSE;
 }
-
-
